# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-05 20:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0011_client_client_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='client_secret',
            field=models.CharField(blank=True, default=b'', max_length=255),
        ),
    ]
