<form method="post" action="{% url 'login' %}">
{% csrf_token %}
<input type="hidden" name="next" value="{{ next }}" />
<input type="text" name="username">
<input type="password" name="password">
<input type="submit" value="Enter">
</form>