<h1>Request for Permission</h1>

<p>Client <strong>{{ client.name }}</strong> would like to access this information of you ...</p>

<form method="post" action="{% url 'oidc_provider:authorize' %}">

    {% csrf_token %}

    {{ hidden_inputs }}

    <ul>
        {% for scope in scopes %}
        <li><strong>{{ scope.name }}</strong> <br><i>{{ scope.description }}</i></li>
        {% endfor %}
    </ul>

    <input type="submit" value="Decline" />
    <input name="allow" type="submit" value="Authorize" />

</form>
