# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-09-12 14:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0017_auto_20160811_1954'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='response_type',
            field=models.CharField(
                choices=[
                    ('client_credentials','client_credentials (user passwd flow)')],
                max_length=30,
                verbose_name='Response Type'),
        ),
    ]
